extension YModel on Map {
  T? byMap<T>(String key, T? Function(Map<dynamic, dynamic> rawValue) onDeserialize, {T? defaultValue}) {
    if (this[key] == null) return defaultValue;
    return onDeserialize.call(this[key]) ?? defaultValue;
  }

  List<T>? byList<T>(String key, T Function(Map<dynamic, dynamic> rawValue) onDeserialize, {List<T>? defaultValue}) {
    if (this[key] == null) return defaultValue;
    if (this[key] is! List) return defaultValue;
    return (this[key] as List).map((e) => onDeserialize.call(e)).toList();
  }

  String? byString(String key, {String? defaultValue}) {
    if (this[key] == null) return defaultValue;
    return this[key]?.toString() ?? defaultValue;
  }

  int? byInt(String key, {int? defaultValue}) {
    if (this[key] == null) return defaultValue;
    return double.tryParse(byString(key)!)?.toInt() ?? defaultValue;
  }

  double? byDouble(String key, {double? defaultValue}) {
    if (this[key] == null) return defaultValue;
    return double.tryParse(byString(key)!) ?? defaultValue;
  }

  bool? byBool(String key, {bool? defaultValue}) {
    if (this[key] == null) return defaultValue;
    return byString(key) == "true";
  }

  T? byEnum<T, E>(String key, T? Function(String? rawValue) onDeserialize, {T? defaultValue}) {
    if (this[key] == null) return defaultValue;
    return onDeserialize.call(this[key]?.toString()) ?? defaultValue;
  }
}

class _DemoModel {
  _DemoSubModel? model_Null;
  late _DemoSubModel model_NotNull;
  List<_DemoSubModel>? list_Null;
  late List<_DemoSubModel> list_NotNull;
  String? string_Null;
  late String string_NotNull;
  int? int_Null;
  late int int_NotNull;
  double? double_Null;
  late double double_NotNull;
  bool? bool_Null;
  late bool bool_NotNull;
  _DemoEnum? enum_Null;
  late _DemoEnum enum_NotNull;

  _DemoModel.fromMap(Map<dynamic, dynamic> map) {
    model_Null = map.byMap("modelNull", (rawValue) => _DemoSubModel.fromMap(rawValue));
    model_NotNull = map.byMap("modelNotNull", (rawValue) => _DemoSubModel.fromMap(rawValue), defaultValue: _DemoSubModel())!;
    list_Null = map.byList("listNull", (rawValue) => _DemoSubModel.fromMap(rawValue));
    list_NotNull = map.byList("listNotNull", (rawValue) => _DemoSubModel.fromMap(rawValue), defaultValue: [])!;
    string_Null = map.byString("stringNull");
    string_NotNull = map.byString("stringNotNull", defaultValue: "")!;
    int_Null = map.byInt("intNull");
    int_NotNull = map.byInt("intNotNull", defaultValue: 0)!;
    double_Null = map.byDouble("doubleNull");
    double_NotNull = map.byDouble("doubleNotNull", defaultValue: 0.0)!;
    bool_Null = map.byBool("boolNull");
    bool_NotNull = map.byBool("boolNotNull", defaultValue: false)!;
    enum_Null = map.byEnum("enumNull", (rawValue) => _DemoEnum.from(rawValue));
    enum_NotNull = map.byEnum("enumNotNull", (rawValue) => _DemoEnum.from(rawValue), defaultValue: _DemoEnum.UnKnown)!;
  }
}

enum _DemoEnum {
  UnKnown,
  Enum1(rawValue: "Enum1"),
  ;

  final String? rawValue;

  const _DemoEnum({this.rawValue});

  static _DemoEnum from(dynamic rawValue) => values.firstWhere((x) => x.rawValue == rawValue, orElse: () => UnKnown);
}

class _DemoSubModel {
  _DemoSubModel();

  _DemoSubModel.fromMap(Map<dynamic, dynamic> map);
}
